@@ -4,7 +4,7 @@
  *   License:	LGPL - See file COPYING.LIB for details.
  *   Author:	Stefan Hundhammer <sh@suse.de>
  *
- *   Updated:	2003-08-26
+ *   Updated:	2005-01-07
  */
 
 
@@ -22,7 +22,7 @@
 #include "kio/job.h"
 #include "kio/netaccess.h"
 
-#define HAVE_STUPID_COMPILER 1
+#define HAVE_STUPID_COMPILER 0
 
 
 using namespace KDirStat;
@@ -35,14 +35,15 @@ KFileInfo::KFileInfo( KDirTree   *	tree,
     , _next( 0 )
     , _tree( tree )
 {
-    _isLocalFile = true;
-    _name	 = name ? name : "";
-    _device	 = 0;
-    _mode	 = 0;
-    _links	 = 0;
-    _size	 = 0;
-    _blocks	 = 0;
-    _mtime	 = 0;
+    _isLocalFile	= true;
+    _isSparseFile	= false;
+    _name	 	= name ? name : "";
+    _device	 	= 0;
+    _mode	 	= 0;
+    _links	 	= 0;
+    _size	 	= 0;
+    _blocks	 	= 0;
+    _mtime	 	= 0;
 }
 
 
@@ -66,13 +67,30 @@ KFileInfo::KFileInfo( const QString &	filenameWithoutPath,
 
     if ( isSpecial() )
     {
-	_size	 = 0;
-	_blocks	 = 0;
+	_size	 	= 0;
+	_blocks	 	= 0;
+	_isSparseFile	= false;
     }
     else
     {
-	_size	 = statInfo->st_size;
-	_blocks	 = statInfo->st_blocks;
+	_size	 	= statInfo->st_size;
+	_blocks	 	= statInfo->st_blocks;
+	_isSparseFile	= isFile() && ( allocatedSize() < _size );
+
+	if ( _isSparseFile )
+	{
+	    kdDebug() << "Found sparse file: " << this 
+		      << "    Byte size: " << formatSize( byteSize() )
+		      << "  Allocated: " << formatSize( allocatedSize() )
+		      << endl;
+	}
+
+#if 0
+	if ( isFile() && _links > 1 )
+	{
+	    kdDebug() << _links << " hard links: " << this << endl;
+	}
+#endif
     }
     
 #if 0
@@ -96,19 +114,30 @@ KFileInfo::KFileInfo(  const KFileItem	* fileItem,
     _device	 = 0;
     _mode	 = fileItem->mode();
     _links	 = 1;
+
     
     if ( isSpecial() )
     {
-	_size	 = 0;
-	_blocks	 = 0;
+	_size	 	= 0;
+	_blocks	 	= 0;
+	_isSparseFile 	= false;
     }
     else
     {
 	_size	 = fileItem->size();
+
+	// Since KFileItem does not return any information about allocated disk
+	// blocks, calculate that information artificially so callers don't
+	// need to bother with special cases depending on how this object was
+	// constructed.
+	
 	_blocks	 = _size / blockSize();
 
 	if ( ( _size % blockSize() ) > 0 )
 	    _blocks++;
+
+	// There is no way to find out via KFileInfo if this is a sparse file.
+	_isSparseFile = false;
     }
 
     _mtime	 = fileItem->time( KIO::UDS_MODIFICATION_TIME );
@@ -133,6 +162,25 @@ KFileInfo::~KFileInfo()
 }
 
 
+KFileSize
+KFileInfo::allocatedSize() const
+{
+    return blocks() * blockSize();
+}
+
+
+KFileSize
+KFileInfo::size() const
+{
+    KFileSize sz = _isSparseFile ? allocatedSize() : _size;
+
+    if ( _links > 1 )
+	sz /= _links;
+
+    return sz;
+}
+
+
 QString
 KFileInfo::url() const
 {
@@ -957,7 +1005,7 @@ KAnyDirReadJob::startReading()
 {
     KURL url( _dir->url() );
 
-    if ( url.isMalformed() )
+    if ( ! url.isValid() )
     {
 	kdWarning() << k_funcinfo << "URL malformed: " << _dir->url() << endl;
     }
@@ -983,7 +1031,7 @@ KAnyDirReadJob::entries ( KIO::Job *			job,
     NOT_USED( job );
     KURL url( _dir->url() );	// Cache this - it's expensive!
 
-    if ( url.isMalformed() )
+    if ( ! url.isValid() )
     {
 	kdWarning() << k_funcinfo << "URL malformed: " << _dir->url() << endl;
     }
@@ -1053,7 +1101,7 @@ KAnyDirReadJob::stat( const KURL & 	url,
 {
     KIO::UDSEntry uds_entry;
 
-    if ( KIO::NetAccess::stat( url, uds_entry ) )	// remote stat() OK?
+    if ( KIO::NetAccess::stat( url, uds_entry, qApp->mainWidget() ) )	// remote stat() OK?
     {
 	KFileItem entry( uds_entry, url,
 			 true,		// determine MIME type on demand
@@ -1082,7 +1130,7 @@ KAnyDirReadJob::owner( KURL url )
 {
     KIO::UDSEntry uds_entry;
 
-    if ( KIO::NetAccess::stat( url, uds_entry ) )	// remote stat() OK?
+    if ( KIO::NetAccess::stat( url, uds_entry, qApp->mainWidget() ) )	// remote stat() OK?
     {
 	KFileItem entry( uds_entry, url,
 			 true,		// determine MIME type on demand
@@ -1497,7 +1545,7 @@ KDirStat::fixedUrl( const QString & dirtyUrl )
 {
     KURL url = dirtyUrl;
 
-    if ( url.isMalformed() )		// Maybe it's just a path spec?
+    if ( ! url.isValid() )		// Maybe it's just a path spec?
     {
 	url = KURL();			// Start over with an empty, but valid URL
 	url.setPath( dirtyUrl );	// and use just the path part.
